@@ -55,6 +55,7 @@ class Help(Operator):
         """Show 'About' dialog."""
         dialog = self.get_widget("about_dialog")
         dialog.run()
+        LOG.debug("Showing 'About' window")
         dialog.hide()
 
 
@@ -70,6 +71,7 @@ class File(Operator):
                 self.new()
         else:
             self.new()
+        self.bus.fire()
 
     def new(self):
         """Make new project."""
@@ -84,6 +86,7 @@ class File(Operator):
         if self.state.project_isaltered:
             return
         self.new()
+        self.bus.fire()
 
     def open(self, fname, merge=False):
         """Load project file."""
@@ -97,6 +100,7 @@ class File(Operator):
                 peak.register_queue(self.bus)
         if not merge:
             self.state.active_spectra = [spectra[idx] for idx in active_idxs]
+            self.bus.fire()
             self.state.current_project = fname
         LOG.info("Opened project file '{}'".format(fname))
 
@@ -116,6 +120,7 @@ class File(Operator):
             CONFIG["IO"]["project-dir"] = dialog.get_current_folder()
             self.open(fname)
         dialog.hide()
+        self.bus.fire()
 
     def on_merge(self, *_args):
         """Merges a project file into the current project."""
@@ -128,6 +133,7 @@ class File(Operator):
             CONFIG["IO"]["project-dir"] = dialog.get_current_folder()
             self.open(fname, merge=True)
         dialog.hide()
+        self.bus.fire()
 
     def on_import(self, *_args):
         """Imports spectra from a data file."""
@@ -144,6 +150,7 @@ class File(Operator):
                 spectrum = self.data.add_spectrum(**specdict)
                 spectrum.register_queue(self.bus)
         dialog.hide()
+        self.bus.fire()
 
     def ask_for_save(self):
         """Opens a AskForSaveDialog and then either saves the file or,
@@ -151,6 +158,7 @@ class File(Operator):
         to False. If the dialog is canceled, nothing happens and
         project_isaltered stays True."""
         dialog = self.get_widget("save_confirmation_dialog")
+        dialog.present_with_time(Gdk.CURRENT_TIME)
         response = dialog.run()
         if response == Gtk.ResponseType.YES:
             dialog.hide()
@@ -160,6 +168,7 @@ class File(Operator):
             self.state.project_isaltered = False
         else:
             dialog.hide()
+        self.bus.fire()
 
     def save(self, fname):
         """Saves project file."""
@@ -174,6 +183,7 @@ class File(Operator):
             self.on_save_as()
         else:
             self.save(self.state.current_project)
+        self.bus.fire()
 
     def on_save_as(self, *_args):
         """Saves the current project as a new file."""
@@ -187,6 +197,7 @@ class File(Operator):
             CONFIG["IO"]["project-dir"] = dialog.get_current_folder()
             self.save(fname)
         dialog.hide()
+        self.bus.fire()
 
     def on_export_txt(self, *_args):
         """Exports the currently selected spectra and their fits to an ASCII
@@ -205,6 +216,7 @@ class File(Operator):
                 gxps.io.export_txt(fname, spectrum)
         CONFIG["IO"]["project-dir"] = dialog.get_current_folder()
         dialog.hide()
+        self.bus.fire()
 
     def on_export_image(self, *_args):
         """Exports the image currently displayed on the exporting canvas.
@@ -242,6 +254,18 @@ class File(Operator):
                 success = True
         dialog.hide()
 
+    def on_change_image_exporter(self, *_args):
+        """Changes the export canvas according to user settings."""
+        canvas = self.get_widget("export_canvas")
+        title = self.get_widget("img_export_title")
+        xlabel = self.get_widget("img_export_xlabel")
+        ylabel = self.get_widget("img_export_ylabel")
+        canvas.ax.set_title(title.get_text())
+        canvas.ax.set_xlabel(xlabel.get_text())
+        canvas.ax.set_ylabel(ylabel.get_text())
+        canvas.draw_idle()
+
+
 
 class Edit(Operator):
     """Contains methods for user initiated data manipulation."""
@@ -250,6 +274,7 @@ class Edit(Operator):
         spectra = self.state.selected_spectra
         for spectrum in spectra:
             self.data.remove_spectrum(spectrum)
+        self.bus.fire()
 
     def on_edit_spectra(self, *_args):
         """Opens an 'Edit' dialog for selected spectra."""
@@ -266,6 +291,7 @@ class Edit(Operator):
                     spectrum.set_meta(attr, value)
         dialog.hide()
         self.state.editing_spectra = []
+        self.bus.fire()
 
     def on_calibrate(self, *_args):
         """Changes the calibration for selected spectra."""
@@ -273,6 +299,7 @@ class Edit(Operator):
         calibration = float(adjustment.get_value())
         for spectrum in self.state.active_spectra:
             spectrum.energy_calibration = calibration
+        self.bus.fire()
 
     def on_normalize(self, *_args):
         """Changes the normalization for selected spectra."""
@@ -283,6 +310,7 @@ class Edit(Operator):
         norm_type = self.state.titles["norm_type_ids"].inverse[normid]
         for spectrum in self.state.active_spectra:
             spectrum.normalization_type = norm_type
+        self.bus.fire()
 
     def on_normalize_manual(self, *_args):
         """Changes the normalization divisor directly."""
@@ -291,6 +319,7 @@ class Edit(Operator):
             if spectrum.normalization_type != "manual":
                 raise ValueError("Normalization is not set to manual")
             spectrum.normalization_divisor = 1 / float(entry.get_text())
+        self.bus.fire()
 
     def on_change_bg(self, *_args):
         """Changes the background type."""
@@ -304,6 +333,7 @@ class Edit(Operator):
         bg_type = self.state.titles["background_type_ids"].inverse[bgid]
         for spectrum in active_spectra:
             spectrum.background_type = bg_type
+        self.bus.fire()
 
 
 class Fit(Operator):
@@ -311,17 +341,20 @@ class Fit(Operator):
     manipulation."""
     def on_add_region(self, *_args):
         """Add two region boundaries to each of the active spectra."""
+        button = self.get_widget("region_add_button")
+        if button.get_active() != True:
+            return
+        navbar = self.get_widget("plot_toolbar")
         def add_region(emin, emax):
             """Add region"""
             for spectrum in self.state.active_spectra:
                 spectrum.background_type = "shirley"
                 spectrum.add_background_bounds(emin, emax)
+            self.bus.fire()
         spanprops = {"edgecolor": COLORS["Plotting"]["region-vlines"], "lw": 2}
-        navbar = self.get_widget("plot_toolbar")
         navbar.get_span(add_region, **spanprops)
 
-    @staticmethod
-    def on_change_region(event):
+    def on_change_region(self, event):
         """Set a new region boundary."""
         if "release" not in event.properties["attr"]:
             return
@@ -336,9 +369,14 @@ class Fit(Operator):
             if old_value == upper:
                 spectrum.remove_background_bounds(lower, upper)
                 spectrum.add_background_bounds(lower, new_value)
+        self.bus.fire()
 
     def on_remove_region(self, *_args):
         """Remove selected region."""
+        button = self.get_widget("region_remove_button")
+        if button.get_active() != True:
+            return
+        navbar = self.get_widget("plot_toolbar")
         def remove_region(_x_0, _y_0, x_1, _y_1):
             """Remove region"""
             esel = x_1
@@ -347,11 +385,17 @@ class Fit(Operator):
                 for lower, upper in zip(bg_bounds[0::2], bg_bounds[1::2]):
                     if esel >= lower and esel <= upper:
                         spectrum.remove_background_bounds(lower, upper)
-        navbar = self.get_widget("plot_toolbar")
+                if not any(spectrum.background_bounds):
+                    spectrum.background_type = "none"
+            self.bus.fire()
         navbar.get_point(remove_region)
 
     def on_add_peak(self, *_args):
         """Add peak to active regions."""
+        button = self.get_widget("peak_add_button")
+        if button.get_active() != True:
+            return
+        navbar = self.get_widget("plot_toolbar")
         shape_combo = self.get_widget("new_peak_model_combo")
         shape = shape_combo.get_active_text()
         def add_peak(position, height, angle):
@@ -362,8 +406,8 @@ class Fit(Operator):
                 peak = spectrum.add_peak(name, position=position, angle=angle,
                                          height=height, shape=shape)
                 peak.register_queue(self.bus)
+            self.bus.fire()
         wedgeprops = {}
-        navbar = self.get_widget("plot_toolbar")
         navbar.get_wedge(add_peak, **wedgeprops)
 
     def on_remove_active_peak(self, *_args):
@@ -371,17 +415,21 @@ class Fit(Operator):
         for peak in self.state.active_peaks:
             self.state.peak_names.remove(peak.name)
             peak.spectrum.remove_peak(peak)
+        self.bus.fire()
 
     def on_clear_peaks(self, *_args):
         """Remove all peaks from active spectra."""
-        # with EventQueue("combine-all"):
         for spectrum in self.state.selected_spectra:
             for peak in spectrum.peaks:
                 self.state.peak_names.remove(peak.name)
                 spectrum.remove_peak(peak.name)
+        self.bus.fire()
 
     def on_peak_entry_activate(self, *_args):
         """Change the active peak's parameters."""
+        self.bus.set_policy("ignore")
+        self.on_peak_model_changed()
+        self.bus.set_policy("accumulate")
         active_peaks = self.state.active_peaks
         if len(active_peaks) != 1:
             return
@@ -399,9 +447,8 @@ class Fit(Operator):
             constraint["param_alias"] = attr
             constraints.append(constraint)
         for constraint in constraints:
-            peak.set_constraint(**constraint)
-        model_combo = self.get_widget("peak_model_combo")
-        peak.shape = peak.shapes[model_combo.get_active()]
+            peak.set_constraints(**constraint)
+        self.bus.fire()
 
     def on_peak_model_changed(self, *_args):
         """Change the model of the active peak."""
@@ -410,8 +457,10 @@ class Fit(Operator):
             return
         peak = active_peaks[0]
         model_combo = self.get_widget("peak_model_combo")
-        shape = model_combo.get_active_text()
+        shape_id = model_combo.get_active_id()
+        shape = self.state.titles["peak_shape_ids"].inverse[shape_id]
         peak.shape = shape
+        self.bus.fire()
 
     def on_peak_name_entry_changed(self, *_args):
         """Change the active peak's name."""
@@ -422,12 +471,14 @@ class Fit(Operator):
         name_entry = self.get_widget("peak_name_entry")
         name = name_entry.get_text()
         peak.label = name
+        self.bus.fire()
 
     def on_fit(self, *_args):
         """Do the fucking fitting."""
         active_spectra = self.state.active_spectra
         for spectrum in active_spectra:
             spectrum.do_fit()
+        self.bus.fire()
 
     @staticmethod
     def parse_peak_entry(param_string):
@@ -456,6 +507,7 @@ class ViewC(Operator):
         """Callback for showing all selected spectra."""
         spectra = self.state.selected_spectra
         self.state.active_spectra = spectra
+        self.bus.fire()
 
     def on_spectrum_view_row_activated(self, treeview, path, _col):
         """Callback for row activation by Enter key or double click."""
@@ -463,6 +515,7 @@ class ViewC(Operator):
         iter_ = model.get_iter(path)
         spectrum = model.get(iter_, 0)[0]
         self.state.active_spectra = [spectrum]
+        self.bus.fire()
 
     def on_spectrum_view_clicked(self, treeview, event):
         """Callback for button-press-event, popups the menu on right click.
@@ -488,6 +541,7 @@ class ViewC(Operator):
             combo.get_active_text(),
             entry.get_text()
         )
+        self.bus.fire()
 
     def on_peak_view_row_activated(self, treeview, path, _col):
         """Callback for row activation by Enter key or single click."""
@@ -495,6 +549,7 @@ class ViewC(Operator):
         iter_ = model.get_iter(path)
         peak = model.get(iter_, 0)[0]
         self.state.active_peaks = [peak]
+        self.bus.fire()
 
     def on_show_rsfs(self, *_args):
         """Opens an RSF dialog."""
@@ -511,6 +566,7 @@ class ViewC(Operator):
         elif response == Gtk.ResponseType.REJECT:
             self.state.rsf_elements = []
         dialog.hide()
+        self.bus.fire()
 
     def on_center_plot(self, *_args):
         """Centers the plot via the navbar command."""
@@ -519,10 +575,16 @@ class ViewC(Operator):
 
     def on_pan_plot(self, *_args):
         """Activates plot panning."""
+        button = self.get_widget("mpl_pan_button")
+        if button.get_active() != True:
+            return
         navbar = self.get_widget("plot_toolbar")
         navbar.pan()
 
     def on_zoom_plot(self, *_args):
         """Activates plot panning."""
+        button = self.get_widget("mpl_zoom_button")
+        if button.get_active() != True:
+            return
         navbar = self.get_widget("plot_toolbar")
         navbar.zoom()
