@@ -42,6 +42,11 @@ class SpectrumContainer(Observable):
         super().__init__(*args, **kwargs)
         LOG.info("{} created".format(self))
 
+    @property
+    def children(self):
+        """Observable children objects."""
+        return self.spectra
+
     @property
     def spectra(self):
         """Returns spectrum objects."""
@@ -76,8 +81,8 @@ class Spectrum(Observable, MetaDataContainer):
     """
     _signals = ("changed-spectrum", "changed-spectrum-meta")
     _required = ("energy", "intensity", "name", "filename", "notes")
-    _bg_types = ("none", "linear", "shirley", "tougaard")
-    _norm_types = ("none", "highest", "high_energy", "low_energy", "manual")
+    bg_types = ("none", "linear", "shirley", "tougaard")
+    norm_types = ("none", "highest", "high_energy", "low_energy", "manual")
 
     def __init__(self, **kwargs):
         if not all(a in kwargs for a in self._required):
@@ -165,10 +170,10 @@ class Spectrum(Observable, MetaDataContainer):
 
     @normalization_type.setter
     def normalization_type(self, value):
-        """Normalization type has to be in self._norm_types."""
+        """Normalization type has to be in self.norm_types."""
         if self._normalization_type == value:
             return
-        if value not in self._norm_types:
+        if value not in self.norm_types:
             raise ValueError("Invalid normalization type '{}'".format(value))
         self._normalization_type = value
         if self._normalization_type != "manual":
@@ -228,8 +233,8 @@ class Spectrum(Observable, MetaDataContainer):
 
     @background_type.setter
     def background_type(self, value):
-        """Only values from self._bg_types are valid."""
-        if value not in self._bg_types:
+        """Only values from self.bg_types are valid."""
+        if value not in self.bg_types:
             raise ValueError("Background type {} not valid".format(value))
         if self._background_type == value:
             return
@@ -258,7 +263,6 @@ class Spectrum(Observable, MetaDataContainer):
             raise ValueError("Background bounds must be pairwise.")
         if not any(value):
             self._background_bounds = np.array([])
-            self.background_type = self._background_type
         for bound in value:
             if bound > self.energy.max() or bound < self.energy.min():
                 raise ValueError("Background bound out of energy range.")
@@ -293,14 +297,19 @@ class ModeledSpectrum(Spectrum):
     """
     _signals = ("changed-fit", "changed-spectrum", "changed-spectrum-meta")
     _required = ("energy", "intensity", "name", "filename", "notes")
-    _bg_types = ("none", "linear", "shirley", "tougaard")
-    _norm_types = ("none", "manual", "highest", "high_energy", "low_energy")
+    bg_types = ("none", "linear", "shirley", "tougaard")
+    norm_types = ("none", "manual", "highest", "high_energy", "low_energy")
 
     def __init__(self, *args, **kwargs):
         self.params = Parameters()
         self._peaks = []
         super().__init__(*args, **kwargs)
 
+    @property
+    def children(self):
+        """Observable children objects."""
+        return self.peaks
+
     @property
     def peaks(self):
         """Returns peaks."""
@@ -325,6 +334,11 @@ class ModeledSpectrum(Spectrum):
             fit = self.model.eval(params=self.params, x=energy)
         return fit
 
+    @property
+    def residual(self):
+        """Returns the fit residual."""
+        return self.intensity - self.background - self.fit
+
     @property
     def model(self):
         """Returns the sum of all peak models."""
@@ -421,6 +435,13 @@ class Peak(Observable):
         "beta": 0,
         "gamma": 0
     }
+    _default_constraints = {
+        "value": None,
+        "vary": True,
+        "min": 0,
+        "max": np.inf,
+        "expr": ""
+    }
     shapes = ["PseudoVoigt", "DoniachSunjic", "Voigt"]
 
     def __init__(
@@ -456,19 +477,19 @@ class Peak(Observable):
             if alpha is None:
                 alpha = 0.5
             self._model = PseudoVoigtModel(prefix="{}_".format(self.name))
-            self._model.set_param_hint("fraction", vary=False, value=alpha)
+            self._model.set_param_hint("fraction", min=0, value=alpha)
         elif self._shape == "DoniachSunjic":
             self.param_aliases["alpha"] = "asym"
             if alpha is None:
                 alpha = 0.1
             self._model = DoniachSunjicModel(prefix="{}_".format(self.name))
-            self._model.set_param_hint("asym", vary=False, value=alpha)
+            self._model.set_param_hint("asym", min=0, value=alpha)
         elif self._shape == "Voigt":
             self.param_aliases["alpha"] = "fwhm_l"
             if alpha is None:
                 alpha = 0.5
             self._model = VoigtModel(prefix="{}_".format(self.name))
-            self._model.set_param_hint("fwhm_l", vary=False, value=alpha)
+            self._model.set_param_hint("fwhm_l", min=0, value=alpha)
         else:
             if "" in (beta, gamma, ):
                 pass #only for linter
@@ -476,8 +497,8 @@ class Peak(Observable):
 
         self.params += self._model.make_params()
         self.get_param("fwhm").set(value=abs(fwhm), min=0, vary=True)
-        self.get_param("amplitude").set(value=abs(area), min=0)
-        self.get_param("center").set(value=abs(position), min=0)
+        self.get_param("amplitude").set(value=abs(area), min=0, vary=True)
+        self.get_param("center").set(value=abs(position), min=0, vary=True)
         # if self._shape == "PseudoVoigt":
         #     self.get_param("sigma").set(expr="{}_fwhm/2".format(self.name))
 
@@ -522,6 +543,8 @@ class Peak(Observable):
     @label.setter
     def label(self, value):
         """Emit a signal when changing the label."""
+        if self._label == value:
+            return
         self._label = value
         self.emit("changed-peak-meta", attr="label", value=value)
 
@@ -541,47 +564,48 @@ class Peak(Observable):
             intensity = self._model.eval(params=self.params, x=energy)
         return intensity
 
-    def set_constraint(
+    def set_constraints(
             self, param_alias,
-            value=None, vary=None, min=None, max=None, expr=None
+            value=None, vary=None, min=0, max=np.inf, expr=""
         ):
         """Sets a constraint for param. None values will unset the constraint.
         """
         # pylint: disable=too-many-arguments
         # pylint: disable=redefined-builtin
-        param = self.get_param(param_alias)
-        old_par = self.get_constraints(param_alias)
-
-        for arg in (value, vary, min, max):
-            try:
-                # fail if x is not in (None, number)
-                if arg and abs(arg) > -1:
-                    pass
-            except TypeError:
-                LOG.warning("Invalid constraint value '{}'".format(arg))
-                raise TypeError("Invalid constraint value '{}'".format(arg))
-
-        if min is None:
-            min = 0
-        if max is None:
-            max = np.inf
         if vary is None:
-            vary = value is None
+            vary = value is None and expr == ""
 
-        param.set(min=min, max=max, vary=vary, value=value, expr="")
+        try:
+            param = self.get_param(param_alias)
+        except ValueError:
+            LOG.debug(f"Skipped parameter {param_alias} (model {self.model})")
+            return
 
-        if expr is not None:
+        new = {
+            "value": value, "vary": vary, "min": min, "max": max, "expr": expr
+        }
+        old = self.get_constraints(param_alias)
+
+        for key, arg in new.items():
+            if arg != old[key] and arg is not None:
+                break
+        else:
+            return
+
+        if expr == "":
+            param.set(**new)
+        else:
             expr = self.relation2expr(expr, param_alias)
             try:
-                param.set(expr=expr, min=-np.inf, max=np.inf)
+                param.set(expr=expr, min=0, max=np.inf)
                 self.params.valuesdict()
             except (SyntaxError, NameError, TypeError):
-                old_par["expr"] = ""
-                param.set(**old_par)
+                old["expr"] = ""
+                param.set(**old)
                 self.emit("changed-peak")
                 LOG.warning("Invalid expression '{}'".format(expr))
 
-        LOG.info("Fit parameter set to '{}'".format(param))
+        LOG.info("Fit parameter set: '{}'".format(param))
         self.emit("changed-peak")
 
     def get_constraints(self, param_alias):
@@ -589,14 +613,7 @@ class Peak(Observable):
         try:
             param = self.get_param(param_alias)
         except ValueError:
-            constraints = {
-                "value": None,
-                "min": -np.inf,
-                "max": np.inf,
-                "vary": None,
-                "expr": ""
-            }
-            return constraints
+            return self._default_constraints
 
         relation = ""
         if param.expr is not None:
@@ -627,7 +644,6 @@ class Peak(Observable):
     def relation2expr(self, relation, param_alias):
         """Translates a human-readable arithmetic relation to an expr string.
         """
-        param_name = self.param_aliases[param_alias]
         def name_repl(matchobj):
             """Replaces 'peakname' by 'peakname_param' (searches
             case-insensitive).
@@ -636,8 +652,11 @@ class Peak(Observable):
             name = name.upper()
             if name == self.name.upper():
                 raise ValueError("Self-reference in peak constraint")
-            if name in [peak.name.upper() for peak in self.spectrum.peaks]:
-                return "{}_{}".format(name, param_name)
+            for peak in self.spectrum.peaks:
+                if peak.name.upper() == name:
+                    other = peak
+                    param_name = other.param_aliases[param_alias]
+                    return "{}_{}".format(name, param_name)
             return name
         regex = r"\b[A-Za-z][A-Za-z0-9]*"
         expr = re.sub(regex, name_repl, relation)
@@ -669,6 +688,12 @@ class Peak(Observable):
             area["value"], fwhm["value"], position["value"],
             alpha["value"], beta["value"], gamma["value"]
         )
+        self.set_constraints("fwhm", **fwhm)
+        self.set_constraints("area", **area)
+        self.set_constraints("position", **position)
+        self.set_constraints("alpha", **alpha)
+        self.set_constraints("beta", **beta)
+        self.set_constraints("gamma", **gamma)
         self.emit("changed-peak")
 
     def get_area(self):
