@@ -14,7 +14,6 @@ from gi.repository import Gtk, Pango, GLib
 from gxps import __appname__
 from gxps.config import COLORS
 from gxps.io import get_element_rsfs
-from gxps.spectrum import Peak
 
 
 LOG = logging.getLogger(__name__)
@@ -43,7 +42,7 @@ class ViewManager():
         window = Window(self.get_widget, self.state, self.data)
         signals = ("changed-project", )
         for signal in signals:
-            self.bus.subscribe(window.update_titlebar, signal)
+            self.bus.subscribe(window.update_titlebar, signal, priority=0)
         return window
 
     def _instantiate_plot(self):
@@ -52,7 +51,7 @@ class ViewManager():
         signals = ("changed-active", "changed-rsf", "changed-spectrum",
                    "changed-fit", "changed-peak")
         for signal in signals:
-            self.bus.subscribe(plot.update, signal, priority=10)
+            self.bus.subscribe(plot.update, signal, priority=2)
         return plot
 
     def _instantiate_spectra_panel(self):
@@ -152,6 +151,7 @@ class Plot(View):
         super().__init__(*args, **kwargs)
         self._canvas = self.get_widget("main_canvas")
         self._ax = self._canvas.ax
+        self._resid_ax = self._canvas.resid_ax
 
     def update(self, event, keepaxes=True):
         """Updates plot by redrawing the whole thing. Relies on
@@ -166,14 +166,19 @@ class Plot(View):
                     keepaxes = False
         if event.signal == "changed-active":
             keepaxes = False
+        self._update(keepaxes)
+
+    def _update(self, keepaxes=True):
         # Save axis limits if needed, wipe the canvas and prepare for new
         # centering axis limits.
         if keepaxes:
             self._canvas.store_xylims()
         self._ax.cla()
+        self._resid_ax.cla()
         self._canvas.reset_xy_centerlims()
         self._plot_spectra()
         self._plot_peaks()
+        self._plot_residual()
         self._plot_rsf()
         # Either restore axis limits or center plot.
         if keepaxes:
@@ -221,6 +226,35 @@ class Plot(View):
                 max(spectrum.intensity)
             )
 
+    def _plot_residual(self):
+        line = {
+            "color": COLORS["Plotting"]["residual"],
+            "linewidth": 1,
+            "linestyle": "-",
+            "alpha": 0.8
+        }
+        residual_exists = False
+        for spectrum in self.state.active_spectra:
+            if (not spectrum.fit.any()
+                    or (spectrum.fit == spectrum.intensity).all()):
+                continue
+            self._resid_ax.plot(
+                spectrum.energy,
+                spectrum.residual,
+                **line
+            )
+            residual_exists = True
+            self._resid_ax.autoscale()
+        if residual_exists:
+            line = {
+                "color": COLORS["Plotting"]["axisticks"],
+                "alpha": 0.5,
+                "linestyle": "--",
+                "linewidth": 0.5
+            }
+            self._resid_ax.axhline(0, **line)
+
+
     def _plot_peaks(self):
         inactive_color = COLORS["Plotting"]["peak"]
         active_color = COLORS["Plotting"]["peak-active"]
@@ -252,6 +286,8 @@ class Plot(View):
                     spectrum.background + peak.intensity,
                     **line
                 )
+            if not any(spectrum.fit):
+                continue
             line = {
                 "color": sum_color,
                 "linewidth": 1,
@@ -302,12 +338,24 @@ class SpectraPanel(View):
     """
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
+
         tvmenu = self.get_widget("spectrum_view_context_menu")
         treeview = self.get_widget("spectrum_view")
         tvmenu.attach_to_widget(treeview, None)
         norm_combo = self.get_widget("normalization_combo")
+        norm_combo.remove_all()
+        for norm_type in self.state.titles["norm_types"]:
+            norm_id = self.state.titles["norm_type_ids"][norm_type]
+            norm_title = self.state.titles["norm_types"][norm_type]
+            norm_combo.insert(-1, norm_id, norm_title)
         renderer = norm_combo.get_cells()[0]
         renderer.set_property("width-chars", 10)
+        bg_type_combo = self.get_widget("region_background_type_combo")
+        bg_type_combo.remove_all()
+        for bg_type in self.state.titles["background_types"]:
+            bg_type_id = self.state.titles["background_type_ids"][bg_type]
+            bg_type_title = self.state.titles["background_types"][bg_type]
+            bg_type_combo.insert(-1, bg_type_id, bg_type_title)
         self._make_columns()
         self._setup_filter()
 
@@ -480,10 +528,13 @@ class PeakPanel(View):
         shape_combo.remove_all()
         model_combo = self.get_widget("peak_model_combo")
         model_combo.remove_all()
-        for shape in Peak.shapes:
-            shape_combo.append_text(shape)
-            model_combo.append_text(shape)
+        for shape in self.state.titles["peak_shapes"]:
+            shape_id = self.state.titles["peak_shape_ids"][shape]
+            shape_title = self.state.titles["peak_shapes"][shape]
+            shape_combo.insert(-1, shape_id, shape_title)
+            model_combo.insert(-1, shape_id, shape_title)
         shape_combo.set_active(0)
+
         self._make_columns()
 
     def update_data(self, _event):
@@ -566,9 +617,10 @@ class PeakPanel(View):
                 entries[par].set_sensitive(False)
                 labels[par].set_text("")
 
-        for i, shape in enumerate(Peak.shapes):
+        # for i, shape in enumerate(Peak.shapes):
+        for shape, shape_id in self.state.titles["peak_shape_ids"].items():
             if shape == peak.shape:
-                model_combo.set_active(i)
+                model_combo.set_active_id(shape_id)
         model_combo.set_sensitive(True)
 
 
